require "import"
import "android.widget.*"
import "com.androlua.*"

-- Layout definition
layout = {
    GridView,
    id = "grid",
    numColumns = 1,
    layout_width = "fill",
    layout_height = "fill"
}

-- Language options
items = {"english", "hindi", "marathi", "urdu", "bengali", "punjabi"}

-- Create a dialog
dlg = LuaDialog(this)
dlg.View = loadlayout(layout)
grid.adapter = SingleLineAdapter(service, String(items))
dlg.setTitle("Welcome to Android Blind World Team")
dlg.setButton("Close", nil)
dlg.show()

-- Common function to start speech recognition
function startSpeechRecognition(languageCode)
    require "import"
    import "android.content.Context"
    import "android.content.Intent"
    import "java.util.Locale"
    import "android.speech.SpeechRecognizer"
    import "android.speech.RecognitionListener"
    import "android.speech.RecognizerIntent"
  
    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)
  
    local recognitionListener = RecognitionListener{
        onResults = function(results)
            local resultArray = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if resultArray ~= nil and resultArray.size() > 0 then
                local recognizedText = resultArray.get(0)
                local editText = service.getEditText()
                if editText then
                    service.insertText(editText, recognizedText .. "۔ ")
                    service.speak(recognizedText)
                else
                    service.speak("Edit box not found")
                end
            end
            destroySpeechRecognizer(speechRecognizer)
        end,
        onError = function()
            destroySpeechRecognizer(speechRecognizer)
            return false
        end
    }
  
    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, languageCode)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())
    speechRecognizer.startListening(recognizerIntent)
    speechRecognizer.setRecognitionListener(recognitionListener)
  
    return true
end

-- Function to clean up the SpeechRecognizer instance
function destroySpeechRecognizer(rec)
    if rec then
        rec.destroy()
        rec = nil
    end
    return true
end

-- Handle item clicks
grid.onItemClick = function(l, v)
    dlg.dismiss()
    task(20, function()
        if v.text == "english" then
            startSpeechRecognition("en-IN")
        elseif v.text == "marathi" then
            startSpeechRecognition("mr-IN")
        elseif v.text == "hindi" then
            startSpeechRecognition("hi-IN")
        elseif v.text == "urdu" then
            startSpeechRecognition("ur-PK")
        elseif v.text == "bengali" then
            startSpeechRecognition("bn-IN")
        elseif v.text == "punjabi" then
            startSpeechRecognition("pa-IN")
        end
    end)
end